package com.landray.kmss.sys.attachment.integrate.wps.authen;

import com.landray.kmss.sys.authentication.util.UUID;
import java.util.Date;
import java.text.DateFormat;
import com.landray.kmss.sys.authentication.ssoclient.Logger;
import java.util.TimeZone;
import java.text.SimpleDateFormat;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.Provider;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import java.security.Security;
import java.security.spec.X509EncodedKeySpec;
import java.security.Key;
import java.security.spec.KeySpec;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.KeyFactory;
import com.landray.kmss.sys.authentication.util.StringUtil;
import org.apache.commons.logging.LogFactory;
import javax.crypto.Cipher;
import org.apache.commons.logging.Log;

public class WpsOfficeLRTokenGenerator extends WpsOfficeTokenGenerator
{
    public static final String SECURITY_KEY_PRIVATE = "security.key.private";
    public static final String SECURITY_KEY_PUBLIC = "security.key.public";
    private static final String DATETIME_FORMAT = "yyyyMMddHHmmss";
    private static final int DATETIME_LENGTH;
    private static final int RANDOM_STRING_LENGTH = 32;
    private static final int KEY_SIZE = 1024;
    private static Log logger;
    private String privateKey;
    private String publicKey;
    private Cipher decryptionCipher;
    private Cipher encryptionCipher;
    
    static {
        DATETIME_LENGTH = "yyyyMMddHHmmss".length();
        WpsOfficeLRTokenGenerator.logger = LogFactory.getLog((Class)WpsOfficeLRTokenGenerator.class);
    }
    
    @Override
    protected void init(final String publicKey, final String privateKey) throws Exception {
        this.setPublicKey(publicKey);
        this.setPrivateKey(privateKey);
    }
    
    private void setPrivateKey(final String privateKey) throws Exception {
        if (StringUtil.isNull(privateKey)) {
            this.privateKey = null;
            this.decryptionCipher = null;
        }
        else {
            this.privateKey = privateKey;
            final KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            final KeySpec keySpec = new PKCS8EncodedKeySpec(StringUtil.toBytes(privateKey));
            final Key priKey = keyFactory.generatePrivate(keySpec);
            (this.decryptionCipher = Cipher.getInstance("RSA/ECB/PKCS1Padding", "BC")).init(2, priKey);
        }
    }
    
    private void setPublicKey(final String publicKey) throws Exception {
        if (StringUtil.isNull(publicKey)) {
            this.publicKey = null;
            this.encryptionCipher = null;
        }
        else {
            this.publicKey = publicKey;
            final KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            final KeySpec keySpec = new X509EncodedKeySpec(StringUtil.toBytes(publicKey));
            final Key pubKey = keyFactory.generatePublic(keySpec);
            (this.encryptionCipher = Cipher.getInstance("RSA/ECB/PKCS1Padding", "BC")).init(1, pubKey);
        }
    }
    
    public void generateKeys() throws Exception {
        if (Security.getProvider("BC") == null) {
            Security.addProvider((Provider)new BouncyCastleProvider());
        }
        final KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");
        keyGen.initialize(1024);
        final KeyPair key = keyGen.generateKeyPair();
        this.setPublicKey(StringUtil.toHexString(key.getPublic().getEncoded()));
        this.setPrivateKey(StringUtil.toHexString(key.getPrivate().getEncoded()));
    }
    
    private synchronized byte[] decrypt(final byte[] input) throws Exception {
        return this.decryptionCipher.doFinal(input);
    }
    
    private synchronized byte[] encrypt(final byte[] input) throws Exception {
        return this.encryptionCipher.doFinal(input);
    }
    
    @Override
    public WpsOfficeToken generateTokenByTokenString(final String tokenString) {
        if (StringUtil.isNull(tokenString) || "\"\"".equals(tokenString)) {
            return null;
        }
        try {
            final byte[] cipherText = StringUtil.toBytes(tokenString);
            String tokenInfo = new String(this.decrypt(cipherText), "UTF-8");
            tokenInfo = tokenInfo.substring(0, tokenInfo.length() - 32);
            final String username = tokenInfo.substring(WpsOfficeLRTokenGenerator.DATETIME_LENGTH, tokenInfo.length() - WpsOfficeLRTokenGenerator.DATETIME_LENGTH);
            final String expireStr = tokenInfo.substring(tokenInfo.length() - WpsOfficeLRTokenGenerator.DATETIME_LENGTH);
            final DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
            dateFormat.setTimeZone(TimeZone.getTimeZone("GMT0"));
            final Date expireTime = dateFormat.parse(expireStr);
            if (Logger.isDebugEnabled()) {
                Logger.debug("成功获取Token信息，用户名：" + username + "，过期时间：" + expireTime);
            }
            return new WpsOfficeToken(expireTime.getTime(), username, tokenString);
        }
        catch (final Exception e) {
            Logger.warn("解密Token信息发生错误：" + tokenString, (Throwable)e);
            return null;
        }
    }
    
    @Override
    public WpsOfficeToken generateTokenByUserName(String username) {
        if (StringUtil.isNull(username)) {
            return null;
        }
        try {
            username = username.trim();
            final DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
            dateFormat.setTimeZone(TimeZone.getTimeZone("GMT0"));
            long time = System.currentTimeMillis();
            String tokenInfo = dateFormat.format(new Date(time));
            tokenInfo = String.valueOf(tokenInfo) + username;
            time += this.getMaxAge() * 1000L;
            tokenInfo = String.valueOf(tokenInfo) + dateFormat.format(new Date(time));
            tokenInfo = String.valueOf(tokenInfo) + UUID.randomUUID().toString().replaceAll("-", "");
            final byte[] cipherText = tokenInfo.getBytes("UTF-8");
            final String tokenString = StringUtil.toHexString(this.encrypt(cipherText));
            return new WpsOfficeToken(time, username, tokenString);
        }
        catch (final Exception e) {
            Logger.warn("加密Token信息发生错误，用户名：" + username, (Throwable)e);
            return null;
        }
    }
    
    public String getPrivateKey() {
        return this.privateKey;
    }
    
    public String getPublicKey() {
        return this.publicKey;
    }
    
    @Override
    protected String getKeyFileString() {
        String rtnVal = "security.key.public=" + this.publicKey + "\r\n";
        rtnVal = String.valueOf(rtnVal) + "security.key.private=" + this.privateKey;
        return rtnVal;
    }
    
    public static void main(final String[] args) throws Exception {
        String localServerName = "http://sdfsdf/asdf.jsp;sdf?sdf";
        if (localServerName.indexOf("?") > 0) {
            localServerName = localServerName.substring(0, localServerName.indexOf("?"));
        }
        if (localServerName.indexOf(";") > 0) {
            localServerName = localServerName.substring(0, localServerName.indexOf(";"));
        }
        WpsOfficeLRTokenGenerator.logger.info((Object)localServerName);
    }
}
